*****************************************
Folder structure:
*****************************************

> simulation\	
	## includes R codes to reproduce the simulation results
> physicians_code\
	## includes Data, R codes and intermediate results to reproduce the results of the opioid
  ---- Data
  ---- RCodes
  ---- Results

*****************************************
Reproducibility of simulation results:
*****************************************

1) From the Zipper Folder "codebase_OpioidPaper", extract and download Codes from Folder: simulation\

2) Download Relevant Data from Dropbox link: (https://www.dropbox.com/scl/fo/qk4mvaxpxm7x2mo2k0vy4/AGwXXyAx-MDRY2nVzD6WTfk?rlkey=xozpb90tj2u51c2g67u5eg7gi&st=m1qq385f&dl=0)

Note: We tried to include the data for Simulation Replication in Zipped folder, but Upon extraction, 
we found some files were losing data. Thus, we have included the Dropbox link for Simulation-related Data

3) Once Simulation Data is downloaded, put the files in the "simulation" folder before running the codes mentioned below.

4) Set the folder "simulation" as the R working directory for generating the following outputs:


**************************************************
Generation of Output Files Related to Simulation: 
**************************************************
Please download all the files .csv files from this Dropbox folder (https://www.dropbox.com/scl/fo/qk4mvaxpxm7x2mo2k0vy4/AGwXXyAx-MDRY2nVzD6WTfk?rlkey=xozpb90tj2u51c2g67u5eg7gi&st=m1qq385f&dl=0) and save them in the codebase_OpioidPaper/simulation/ folder. 
These are our pre-run results of the simulation and are provided separately since regenerating them may take some time, while we also provide codes to regenerate these results. The tables and figures of our simulation that we include in our paper may be generated by processing these results files. 

Point 2) below discusses the codes that should be run to regenerate those simulation-results files provided in the Dropbox folder 
Point 1) mentions the codes to create the tables and figures from the simulation-results files provided in the Dropbox folder. 
Codes in 1) may be run without running those in 2).

1) To create the simulation tables and plots from the saved results

	- l1penalty_results.R to create comparison table between L1 penalty
		and our PSC (Supplement Table S1)
		
	- lambda_justification_plot.R to create the plot justifying the 	
		cross-validated selection of Lambda  (Supplement Figure S2)
		
	- calibrated_simulation_results.R to compile and create the plot
		comparing the calibrated simulation study (Supplement Figure S3)
		
	- create_tablev2.R to create the simulation comparison tables
		Create Tables 2 and 3
	
	- create_plot.R to create simulated data example. Figure 4 (note that
		a seed is not set. So, the plot may be different due to randomness.
	
	- create_illustration.R to create Supplement Figure S1
	
2) To run simulation code from scratch and save simulation results

 These call the other R files. So, these are the only code 
 that needs to be run. 

 To run this code, run on terminal using the following bash command
	for ID in {1..180}
	do
		Rscript --vanilla run_sc_simulation_comparison_lambda1000.R $ID
	done
 Note: you can run the above parallelly.

	- run_sc_simulation_comparison_l1penalty_largesd.R 
		run with arguments 1 through 300
			for the comparison of l1 to the proposed penalty 
			
	- run_sc_simulation_comparison_lambda1000.R
		run with arguments 1 through 180
			for lambda cv justification results
			
	- run_sc_simulation_comparison_calibratedv3.R
	  run_sc_simulation_comparison_calibrated_othermodels.R
		run with arguments 1 through 580
			for calibrated simulation study in the supplement
			
	- run_sc_simulation_comparison.R
		run with arguments 1 through 180
			for original simulation study of Section 4
		
*****************************************************
Physician-related Raw Data Access and Post-processing
*****************************************************
This paper uses publicly available data from multiple sources. 

1.	Physician Payment Data:
The dataset on direct payments to physicians is available through the Physician Open Payment Data from the Centers for Medicare and Medicaid Services (https://www.cms.gov/openpayments/data). Data before 2017 are archived and are available at https://www.cms.gov/priorities/key-initiatives/open-payments/data/archived-datasets. From this link, the datasets corresponding to the years 2014 through 2016 can be downloaded. Note, in downloading corresponding to each year, choose the one that starts with OP_DTL_GNRL_PGYR*.
So, there might be minor differences across datasets depending on the download date if CMS updates the datasets. In our analyses, for years 2014, 2015, and 2016, we used the data files OP_DTL_GNRL_PGYR2014_P01212022.csv, OP_DTL_GNRL_PGYR2015_P01212022.csv and OP_DTL_GNRL_PGYR2016_P01212022.csv respectively.
For years 2017 and 2018, data are available at https://www.cms.gov/priorities/key-initiatives/open-payments/data/dataset-downloads. From this link, the datasets corresponding to the years 2017 and 2018 can be downloaded. Note, in downloading corresponding to each year, should choose the one that starts with OP_DTL_GNRL_PGYR*. 
Note that CMS updates data sets. So, there might be minor differences across datasets depending on the download date. In our analyses, for years 2017 and 2018, we used the data files OP_DTL_GNRL_PGYR2017_P01212022.csv, OP_DTL_GNRL_PGYR2018_P01212022.csv respectively.

2.	Opioid Data:
The Physician Payment data provides names of therapeutic drugs that may have been promoted during each payment. Note that the number of drugs promoted during each payment could be one to five. The names of these drugs were manually collected. An RA searched on Google to identify which of these drugs were opioids. Subsequently, the veracity of the information was triangulated through several websites (including but not restricted to www.uptodate.com, https://nida.nih.gov/, https://www.fda.gov/drugs/information-drug-class/opioid-medications, https://www.dea.gov/). Through this process, opioids promoted were identified for each year. 
We have shared those datasets as OpioidNamesXXXX.csv, where XXXX stands for the years 2014-2017.
Opioids from these years are compiled in the shared OpioidMasterList.csv.

3.	Prescription Data: 
We further use annual data on prescriptions for each physician from the Medicare Part D Prescriber Public Use File to find opioid related prescription volumes. The specific dataset used is called “Medicare Part D Prescribers - by Provider and Drug” and can be downloaded from https://data.cms.gov/resources/medicare-part-d-prescribers-by-provider-and-drug-data-dictionary.

4.	Florida Marijuana Patient Registration Data:
These datasets are available from the webpage https://knowthefactsmmj.com/2018/07/28/2017-ommu-updates-archive/. We accessed all web links from this page for the period between June 7, 2017, and December 22, 2017, and manually collected data on marijuana patient counts and dispensary locations over time. Each time period for which the data is provided is considered as a "Period" in our analysis.

*****************************************	
Reproducibility of physician results:
*****************************************

1) From the Zipper Folder "codebase_OpioidPaper", extract and download Data and Code use Folder: physicians_code\
2) Set the folder "physicians_code" as the R working directory for generating the following outputs: 

**************************************************
Generation of Output Files Related to Physicians: 
**************************************************
 
1) To create the tables and plots from the saved results

	- plot_paymentsv2.R to create Table 1 and Figure 2
	
	- FloridaRegressionAnalysis.R to create Table 4
	
	- OpioidPaper_Figure1.R to create Figure 1
	
	- OpioidPaper_Figure3.R to create Figure 3
	
	- plot_balance_diagnostics_72423v1.R	creates the standardized diff
		plot for the synthetic counterpart matches. Supplement Figure S4
	
	- heterogeneity_results_01_26v1.R creates Figures 8, 9 and 10
		corresponding to the heterogeneity analysis and the p-values
	
	- fl_results_v1.R	to creates Figure 6
	
	- see_results_08_19_addCI_blockboot_v4v1.R creates Figure 5
	
	- two_way_fixed_effects_excluded_docs.R creates supplement Table S2: Two-way fixed effects model for the excluded units 	
	- plot_fl_by_dispensary_cities.R creates Figure 7
	
	- plot_balance_vs_effectv1. r  creates Figure S5: Relationship between the estimated effect sizes and the level of pre-treatment fit
	
	- IncludingAllDonorStatesFinal.R  creates Table S3: Average effect of MML enactment including all donor states
	
	
2) To run data analysis code from scratch and save analysis results

 To run this code, run on terminal using the following bash command
	for ID in {1..28}
	do
		Rscript --vanilla run_sync_ctrl_pay_docspl_stv1_intercept_woFL2_FINALRES_blockboot.R $ID
	done

 Note: you can run the above parallelly.
 
	- Rscript --vanilla fl_test_save_each_dropst.R
		