## Implementation of the proposed method
## Setup 
X1 = t(Y[trtid,1:T0]) #as.matrix(d_trt_b[1:10,])
X0 = t(Y[-trtid,1:T0])

N1 = ncol(X1)
N0 = ncol(X0)
X0_int = cbind(X0, 1)
N0_int = ncol(X0_int)

#########################################################################################

## Create the Pmat and qvec for our optimization
## X1 is treated units data matrix, pretreatment
## X0 is control units data matrix, pretreatment
getPmatqvec <- function(X0, X1){

	N1 = ncol(X1)
	X0 = cbind(X0, 1)
	N0 = ncol(X0)
	T = nrow(X1)

	## Separate and no lambda
	qvecfull = NULL
	Pmat = list()
	for(i in 1:ncol(X1)){

		X1i = matrix(X1[,i],ncol=1)
			
		X0i = t(X0[,,drop=FALSE])

		Pmat[[i]] <- X0i %*% t(X0i)
		qvecfull <- c(qvecfull, - t(X1i) %*% t(X0i))
	}

	Pmatfull = Matrix::bdiag(Pmat)


	## Pooled

	qvecpooled <- rep(NA, length(qvecfull))
	for(j in 1:ncol(X0))
		qvecpooled[seq(j,length(qvecpooled), by=N0)] =  
				- mean(sapply(1:T, function(s) sum(X1[s,])*X0[s,j]))
	temp = 0
	for(s in 1:T)
		temp = temp+outer(as.numeric(X0[s,]),as.numeric(X0[s,]))
	temp = temp/T
	dim(temp)
	
	Pmatpooled = matrix(NA, N1*N0, N1*N0)
	for(i in 1:N1)
		for(i1 in 1:N1)
			Pmatpooled[(i-1)*N0+(1:N0),(i1-1)*N0+(1:N0)] = temp

	
	return(list(Pmatfull =Pmatfull, qvecfull=qvecfull, Pmatpooled=Pmatpooled, qvecpooled =qvecpooled))

}

###################################################
res = getPmatqvec(X0, X1)

## setup constraints 

i_sm <- NULL
j_sm <- NULL
for(i in 1:N1){
	i_sm = c(i_sm, rep(i, N0))
	j_sm = c(j_sm, 1:N0)
}
for(i in 1:N1){
	i_sm = c(i_sm, N1+(i-1)*N0+(1:N0))
	j_sm = c(j_sm, (i-1)*N0_int+(1:N0))
}
Afinal1 = Matrix::sparseMatrix(i=i_sm, j=j_sm, x=1, dims=c(N1+N1*N0, N1*N0_int))
gc()
	

l <- c(rep(1,N1), numeric(N1*N0))
u <- c(rep(1,N1), rep(1, N1*N0))

## Run convex optimization to find the sc weights
settings = osqp::osqpSettings(verbose = FALSE, eps_rel = 1e-8, eps_abs = 1e-8)

sol <- try( osqp::solve_osqp(P = res$Pmatfull, q = res$qvecfull,
			A = Afinal1, l = l, u = u, pars = settings) , silent = TRUE)
			
if(class(sol) != "try-error"){
	wmat_sep = matrix(sol$x, ncol=N0+1, byrow=TRUE)
} else sol <- osqp::solve_osqp(P = res$Pmatfull, q = res$qvecfull,
			A = Afinal1, l = l, u = u, pars = settings)

###################################################
#wmat_sep = matrix(sol$x, ncol=N0+1, byrow=TRUE)
dim(wmat_sep)
## This part calculates nu value
## Pooled
sol <- try( osqp::solve_osqp(P = res$Pmatpooled, q = res$qvecpooled,
			A = Afinal1, l = l, u = u, pars = settings), silent = TRUE)

if(class(sol) != "try-error"){
wmat_pooled = matrix(sol$x, ncol=N0+1, byrow=TRUE)
dim(wmat_pooled)

q_sep = mean(sapply(1:N1, function(i) 
			mean( (X1[,i] - wmat_sep[i,]%*%t(X0_int[,,drop=FALSE]))^2 ) ))

q_pool = mean( sapply(1:10, function(s) ( mean(X1[s,]) - mean(wmat_pooled%*%t(X0_int[s,,drop=FALSE])) )^2 ))

## Calculation of nu

nu = min(1, sqrt(q_pool)/sqrt(q_sep))
} else nu = 0

rm(Afinal1)
gc()


###############################################
###############################################

## Function to get cross-validated lambda value

foo_cv <- function(lambda = .1, X1temp, X0temp){

	q_sep_cv = rep(NA, T0)

	for(dropt in 1:T0){

	restemp = getPmatqvec(X0temp[-dropt,], X1temp[-dropt,])
	gc()
	
	N1temp = ncol(X1temp)
	N0temp = ncol(X0temp)
	X0_inttemp = cbind(X0temp, 1)
	N0_inttemp = N0temp+1

	for(i in 1:ncol(X1temp)){
		X1itemp = matrix(X1temp[-dropt,i],ncol=1)
			
		X0itemp = t(X0_inttemp[-dropt,,drop=FALSE])
		zeros = which(X1itemp==0)
		if(length(zeros ) >0)
			restemp$qvecfull[(i-1)*N0_inttemp+(1:N0_inttemp)] = restemp$qvecfull[(i-1)*N0_inttemp+(1:N0_inttemp)] + 
				apply(X0itemp, 1, function(x) sum(exp(lambda*x[zeros])))
	}

	
	Pmataggregatedtemp = restemp$Pmatfull*(1-nu) + restemp$Pmatpooled*nu
	qvecaggregatedtemp = restemp$qvecfull*(1-nu) + restemp$qvecpooled*nu

	
	i_sm <- NULL
	j_sm <- NULL
	for(i in 1:N1temp){
		i_sm = c(i_sm, rep(i, N0temp))
		j_sm = c(j_sm, 1:N0temp)
	}
	for(i in 1:N1temp){
		i_sm = c(i_sm, N1temp+(i-1)*N0temp+(1:N0temp))
		j_sm = c(j_sm, (i-1)*N0_inttemp+(1:N0temp))
	}
	Afinal1 = Matrix::sparseMatrix(i=i_sm, j=j_sm, x=1, dims=c(N1temp+N1temp*N0temp, N1temp*N0_inttemp))

	l <- c(rep(1,N1temp), numeric(N1temp*N0temp))
	u <- c(rep(1,N1temp), rep(1, N1temp*N0temp))


	soltemp <- osqp::solve_osqp(P = Pmataggregatedtemp, q = 				qvecaggregatedtemp,
					A = Afinal1, l = l, u = u, 
					pars = settings)
	

	rm(Afinal1)
	gc()
	
	q_sep_cv[dropt] = mean(sapply(1:N1temp, function(i) 
				(X1temp[dropt,i] - soltemp$x[(i-1)*N0_inttemp+(1:N0_inttemp)]%*%t(X0_inttemp[dropt,,drop=FALSE]))^2 ) )
	}

	mean(q_sep_cv)
	#return(Inf)

}

### Calculate cross-validated lambda

lambdaini = 1/quantile(unlist(X0),.75)
ct = 1
while(ct<10){
getlambda = try(optim(par=lambdaini, fn=foo_cv, #method = "L-BFGS-B",
			lower=.01, upper=50000, X1temp=X1, X0temp=X0), silent=TRUE)
	if(!inherits(getlambda, "try-error")) break;
	ct = ct+1
	#print(ct)
}
if(ct==10){
	lambda = lambdaini
} else lambda = getlambda$par#/100


###################################################
###############################################
###############################################
## Set up and run the optimization
res1 = res
res

for(i in 1:N1){
	X1i = matrix(X1[,i],ncol=1)
		
	X0i = t(X0_int[,,drop=FALSE])
	zeros = which(X1i==0)
	if(length(zeros ) >0)
		res1$qvecfull[(i-1)*N0_int+(1:N0_int)] = res1$qvecfull[(i-1)*N0_int+(1:N0_int)] + 
			apply(X0i, 1, function(x) sum(exp(lambda*x[zeros])))
}

## Pmat and qvec calculation

Pmataggregated = res1$Pmatfull*(1-nu) + res1$Pmatpooled*nu
qvecaggregated = res1$qvecfull*(1-nu) + res1$qvecpooled*nu

## Calculation of the constraints

i_sm <- NULL
j_sm <- NULL
for(i in 1:N1){
	i_sm = c(i_sm, rep(i, N0))
	j_sm = c(j_sm, 1:N0)
}
for(i in 1:N1){
	i_sm = c(i_sm, N1+(i-1)*N0+(1:N0))
	j_sm = c(j_sm, (i-1)*N0_int+(1:N0))
}
Afinal1 = Matrix::sparseMatrix(i=i_sm, j=j_sm, x=1, dims=c(N1+N1*N0, N1*N0_int))
gc()

## Run convex optimization
sol <- osqp::solve_osqp(P = Pmataggregated, q = qvecaggregated,
			A = Afinal1, l = l, u = u, pars = settings)

wmat = matrix(sol$x, ncol=N0_int, byrow=TRUE)
dim(wmat)

###############################################
###############################################
###############################################
## Estimation and rmse calculation

q_sep = mean(sapply(1:N1, function(i) 
			mean( (X1[,i] - sol$x[(i-1)*N0_int+(1:N0_int)]%*%t(X0_int[,,drop=FALSE]))^2 ) ))
#q_sep

X0_all = as.matrix(t(Y[-trtid,]))
X0_all_int = cbind(t(Y[-trtid,]), 1)

## Imbalance
imb = t(Y[trtid,1:T0]) - sapply(1:N1, function(i)
	sol$x[(i-1)*N0_int+(1:N0_int)]%*%t(X0_all_int[1:T0,,drop=FALSE]))
## Estimate
est = t(Y[trtid,(T0+1):T]) - sapply(1:N1, function(i)
	sol$x[(i-1)*N0_int+(1:N0_int)]%*%t(X0_all_int[(T0+1):T,,drop=FALSE]))


l2imb_propsc = mean(apply(imb, 2, function(x) sqrt(mean(x^2))))

## RMSE calculation
esterr = est - est0
rmseitt_propsc = mean(apply(esterr, 2, function(x) sqrt(mean(x^2))))
rmseatt_propsc = sqrt(mean(colMeans(esterr)^2))

res_propsc =c(l2imb_propsc, rmseitt_propsc, rmseatt_propsc)

