## Code to simulate control potential outcomes

# Model parameter values. Fixed throughout the model
# See simulation model in Section 4
rm(list=ls())
b0=20;
b1=1;
b2=2;

T=100;
t=seq(1,T-1,1);
n=length(t);
#-------------------------------------
#-------------------------------------
#-------------------------------------
#-------------------------------------
## Function to generate the model
#	b0 is intercept of z_it, b1 and b2 are coefs
gen<-function(b0,b1,b2,t){
  n=length(t);
  noise=rnorm(n);
  t2=(t-1)%%4;
  t1=(t-1)%/%4
  z=b0+b1*t1+b2*t2+5*noise;
  return(z);
}
#-------------------------------------
#-------------------------------------
#-------------------------------------
#-------------------------------------
d=t;

for (i in 2:n){
  r=runif(1);
  if(d[i-1]==1){
    d[i]=0;
    if(r<=0.9){d[i]=1}
  }
  if(d[i-1]==0){
    d[i]=0;
    if(r<=0.5){d[i]=1}
  }
}

d1=(seq(1,T-1,1)*0.5+40)*(d-1);
#-------------------------------------
d=t;

for (i in 2:n){
  r=runif(1);
  if(d[i-1]==1){
    d[i]=0;
    if(r<=0.9){d[i]=1}
  }
  if(d[i-1]==0){
    d[i]=0;
    if(r<=0.5){d[i]=1}
  }
}

d2=(seq(1,T-1,1)*0.5+40)*(d-1);
#-------------------------------------
d=t;

for (i in 2:n){
  r=runif(1);
  if(d[i-1]==1){
    d[i]=0;
    if(r<=0.9){d[i]=1}
  }
  if(d[i-1]==0){
    d[i]=0;
    if(r<=0.5){d[i]=1}
  }
}

d3=(seq(1,T-1,1)*0.5+40)*(d-1);


#-------------------------------------
#-------------------------------------
#-------------------------------------
#-------------------------------------
## Generate base outcomes for three groups/clusters
k=5;
b01=runif(k,10,60)
b02=runif(k,10,60)
b03=runif(k,10,60)
Z=matrix(0,3*k,n);
for (i in 1:k){Z[i,]=gen(b01[i],b1,b2,t);}
for (i in 1:k){Z[k+i,]=gen(b02[i],b1,b2,t);}
for (i in 1:k){Z[2*k+i,]=gen(b03[i],b1,b2,t);}


Y=(Z+abs(Z))/2;
matplot(t(Y)[,1:5],type="l",col="green",xlab="",ylab="")
matplot(t(Y)[,6:10],type="l",col="red",add=T)
matplot(t(Y)[,11:15],type="l",col="blue",add=T)



d1=matrix(0,1,n);
d2=matrix(0,1,n);
d3=matrix(0,1,n);

d1[c(20:24,70:74)]=matrix(-80,1,10);
d2[c(30:34,80:84)]=matrix(-80,1,10);
d3[c(40:44,90:94)]=matrix(-80,1,10);


Z[1:k,]=Z[1:k,]+matrix(d1,nrow=k,ncol=99,byrow=T);
Z[(k+1):(2*k),]=Z[(k+1):(2*k),]+matrix(d2,nrow=k,ncol=99,byrow=T);
Z[(2*k+1):(3*k),]=Z[(2*k+1):(3*k),]+matrix(d3,nrow=k,ncol=99,byrow=T);


Y=(Z+abs(Z))/2;
matplot(t(Y)[,1:5],type="l",col="green",xlab="",ylab="")
matplot(t(Y)[,6:10],type="l",col="red",add=T)
matplot(t(Y)[,11:15],type="l",col="blue",add=T)

#y=z+d1
#l1=min(y);
#l2=max(z);
#plot(t,z,ylim=c(l1,l2),'l')
#points(t,y,'l',lty=2)
#points(t,(y+abs(y))/2,'l',col="red");


plot(1:99,Y[1,],ylim=c(-100,max(Y[1,])),xlab="",ylab="")
points(1:99,d1,'l',col="red",lty=2)
points(1:99,Z[1,]-d1,'l',col='blue',lty=2)
