## Code to simulate control potential outcomes

# Model parameter values. Fixed throughout the model
# See simulation model in Section 4

#b0=20;	 	
b1=1;
b2=2;

T=100;
t=seq(1,T-1,1);
n=length(t);
#-------------------------------------
#-------------------------------------
#-------------------------------------
#-------------------------------------
## Function to generate the model
#	b0 is intercept of z_it, b1 and b2 are coefs
gen<-function(b0,b1,b2,t){
  n=length(t);
  noise=rnorm(n);
  t2=(t-1)%%4;
  t1=(t-1)%/%4
  z=b0+b1*t1+b2*t2+5*noise;
  return(z);
}
#-------------------------------------
#-------------------------------------
#-------------------------------------
#-------------------------------------
## Generate base outcomes for three groups/clusters
k=10;
b01=runif(k,10,60)
b02=runif(k,10,60)
b03=runif(k,10,60)
Z=matrix(0,3*k,n);
for (i in 1:k){Z[i,]=gen(b01[i],b1,b2,t);}
for (i in 1:k){Z[k+i,]=gen(b02[i],b1,b2,t);}
for (i in 1:k){Z[2*k+i,]=gen(b03[i],b1,b2,t);}


## Different clustering/damping patterns
stickytimes = list()

if(similarclus!=2){
similarclus = similarclus==1
if(!similarclus)
	rates = c(1/10, 1/15, 1/7)
if(similarclus)
	rates = c(1/10, 1/10, 1/10)

stickyrates = c(1/3, 1/3, 1/3)

for(i in 1:3){
	now=1
	stickytimes[[i]] = numeric(0)
	while(now < T-1){
		now = now + floor(rexp(1, rates[i]))
		if(now>T-1) break;
		stick = ceiling(rexp(1, stickyrates[i]))

		stickytimes[[i]] = c(stickytimes[[i]], now:min(T-1,now+stick))
		#print(now:min(T,now+stick))
		now = min(T-1,now+stick)
	}
}
} else {
	stickytimes[[1]] = c(20:24,70:74-30)
	stickytimes[[2]] = c(30:34,80:84-35)
	stickytimes[[3]] = c(40:44,90:94-40)
}

d1=matrix(0,1,n);
d2=matrix(0,1,n);
d3=matrix(0,1,n);

d1[stickytimes[[1]]]=matrix(-80,1,length(stickytimes[[1]]));
d2[stickytimes[[2]]]=matrix(-80,1,length(stickytimes[[2]]));
d3[stickytimes[[3]]]=matrix(-80,1,length(stickytimes[[3]]));

## Add damping seq to the base outcomes

Z[1:k,]=Z[1:k,]+matrix(d1,nrow=k,ncol=99,byrow=T);
Z[(k+1):(2*k),]=Z[(k+1):(2*k),]+matrix(d2,nrow=k,ncol=99,byrow=T);
Z[(2*k+1):(3*k),]=Z[(2*k+1):(3*k),]+matrix(d3,nrow=k,ncol=99,byrow=T);

## Control potential outcomes
Y=(Z+abs(Z))/2;
