# create comparison table between L1 penalty and our PSC
##################

## Reads files paste0('sc_sim_comparison_id_',id,'_null_revisionl1penalty_largesdv8.csv') for id = 1...100
# These files are created by the code
#		run_sc_simulation_comparison_l1penalty_largesd.R

## simulation settings
#	similarclusseq for three clustering models for damping
#		= 2 for deterministically 
#		= 1 for probabilistially similar clusters
#		= 0 for probabilistially dissimilar clustering
## seedseq is used to track parallel run of the code
##
similarclusseq = c(0, 1, 2)		
heteroseq = c(TRUE, FALSE)
seedseq = c(1234, 8041, 1546, 1362, 2315, 2141, 5362, 6163, 2326, 6126)
seedseq = c(seedseq, c(123, 804, 154, 162, 231, 241, 536, 163, 326, 126))
seedseq = c(seedseq, c(13, 84, 54, 62, 31, 41, 53, 63, 36, 26))
seedseq = c(seedseq, c(3213, 584, 1254, 5363, 6431, 7841, 5573, 2363, 6436, 2726))
seedseq = c(seedseq, c(8913, 9884, 8954, 9862, 8931, 9841, 8953, 9863, 8936, 9826))


foo <- function(x, y) c(x, y)
setup = outer(similarclusseq, heteroseq, FUN = Vectorize(foo, SIMPLIFY = FALSE))
setup = outer(setup, seedseq, FUN = Vectorize(foo, SIMPLIFY = FALSE))
setup = sapply(setup, cbind)
###############################################

## Read results from files
res = list()
for(set in 1:6)
	res[[set]] = matrix(0,ncol=13+3,nrow=0)

for(id in 1:300){
	
	similarclus = setup[1,id] + 1 
	hetero = 1+(setup[2,id]==1)
	set 	= 2*(similarclus-1)+ hetero
	print(set)
	f = 	paste0('sc_sim_comparison_id_',id,'_null_revisionl1penalty_largesdv8.csv')
	if(file.exists(f)){
		print('yes')
		temp = as.matrix(read.csv(f, header=TRUE))
		names(temp) = NULL
		res[[set]] = rbind(res[[set]], temp)
	}

}


#################################################

## For data cleaning by removing outliers
rm_outliers <- function(xtemp){
	uq = quantile(xtemp, .975, na.rm = TRUE)
	lq = quantile(xtemp, .005, na.rm = TRUE)
	xtemp_out1 = which(xtemp<=lq | xtemp >=uq)
	xtemp_out2 = which(xtemp %in% boxplot.stats(xtemp)$out)
	
	xtemp_out = intersect(xtemp_out1, xtemp_out2)
	xtemp[-xtemp_out]
}



## mean calculation after removing outliers
mean_rm_outliers <- function(xtemp){
	uq = quantile(xtemp, .975, na.rm = TRUE)
	lq = quantile(xtemp, .0, na.rm = TRUE)
	xtemp_out1 = which(xtemp<=lq | xtemp >=uq)
	xtemp_out2 = which(xtemp %in% boxplot.stats(xtemp)$out)
	
	xtemp_out = intersect(xtemp_out1, xtemp_out2)
	if(length(xtemp[-xtemp_out]) > 0) return( mean(xtemp[-xtemp_out], na.rm=TRUE))
	mean(xtemp, na.rm=TRUE)
}
#################################################

## Summarize results
res_summary = t( rbind(round(t(sapply(res, function(x) apply(x, 2, mean_rm_outliers)))[c(1,3,5),],2), 
round(t(sapply(res, function(x) apply(x, 2, function(y) sd(rm_outliers(y), na.rm=TRUE)/sqrt(length(rm_outliers(y))))))[c(1,3,5),],2)))[-c(1:4),-c(3,6)]

colnames(res_summary) = c('est prob similar clus',
							'est prob dissimilar clus',
							'se prob similar clus',
							'se prob dissimilar clus')
rownames(res_summary) = as.vector(sapply(c('SC', 'PooledSC',
			'PenalizedSC', 'L1PenalizedSC'), paste0, '_',
				c('l2imbalance', 'rmseitt', 'rmseate')))
				
res_summary