## Validate CV choice of lambda
library('matrixStats')

########################################
## simulation settings
#	similarclusseq for three clustering models for damping
#		= 2 for deterministically 
#		= 1 for probabilistially similar clusters
#		= 0 for probabilistially dissimilar clustering
## seedseq is used to track parallel run of the code
similarclusseq = c(0, 1, 2)
heteroseq = c(TRUE, FALSE)
seedseq = c(1234, 8041, 1546, 1362, 2315, 2141, 5362, 6163, 2326, 6126)
seedseq = c(seedseq, c(123, 804, 154, 162, 231, 241, 536, 163, 326, 126))
seedseq = c(seedseq, c(13, 84, 54, 62, 31, 41, 53, 63, 36, 26))
#seedseq = c(seedseq, c(3213, 584, 1254, 5363, 6431, 7841, 5573, 2363, 6436, 2726))
#seedseq = c(seedseq, c(8913, 9884, 8954, 9862, 8931, 9841, 8953, 9863, 8936, 9826))

foo <- function(x, y) c(x, y)
setup = outer(similarclusseq, heteroseq, FUN = Vectorize(foo, SIMPLIFY = FALSE))
setup = outer(setup, seedseq, FUN = Vectorize(foo, SIMPLIFY = FALSE))
setup = sapply(setup, cbind)
########################################

## Read and organize results
res = list()
for(set in 1:6)
	res[[set]] = matrix(0,ncol=13+10*3,nrow=0)

for(id in 1:180){
	
	similarclus = setup[1,id] + 1 
	hetero = 1+(setup[2,id]==1)
	set 	= 2*(similarclus-1)+ hetero
	f = 	paste0('sc_sim_comparison_id_',id,'_null_revisionv2_lambda1000.csv')
	if(file.exists(f)){
		temp = as.matrix(read.csv(f, header=TRUE))
		names(temp) = NULL
		res[[set]] = rbind(res[[set]], temp)
	}

}


res_summary = t( rbind(round(t(sapply(res, colMeans))[c(2,4,6),],2), 
round(t(sapply(res, function(x) apply(x, 2, function(y) sd(y)/sqrt(length(y)))))[c(2,4,6),],2)))

############################ Calculations
itt_difflambdas = res_summary[paste0('X.',seq(8, 36, by=3)),]

itt_difflambdas = itt_difflambdas[c(2:6,1,7:10),]

par(mfcol=c(1,2), oma=c(1.5,1,0,0), mar=c(3,3,2,1.5), las=1)


base = itt_difflambdas[6,]
itt_difflambdas = t(apply(itt_difflambdas, 1, function(x) x/base))

## Plotting
plot(itt_difflambdas[,1], x=c(.1, .2, .25, .5, .75, 1, 1.25, 1.5, 2, 2.5), type='l', lty=2, lwd=2, xaxt='n', ylim=c(1,2.5))
axis(side=1,at=c(.1, .5, .75, 1, 1.25, 1.5, 2, 2.5),labels=c(.1, .5, .75, 1, 1.25, 1.5, 2, 2.5))
mtext(expression(lambda/lambda[cv]),line=3,side=1,cex=1.5)
par(las=0)
mtext(expression(paste("Ratio of RMSEs for ITT for ", lambda," over ",lambda[cv])),
		line=2.5,side=2,cex=1.25)

#itt_difflambdas = res_summary[paste0('X.',seq(8, 36, by=3)),]
points(itt_difflambdas[,2], x=c(.1, .2, .25, .5, .75, 1, 1.25, 1.5, 2, 2.5),  type='l', lwd=2, xaxt='n', lty=1, ylim=c(1,2.5))
points(itt_difflambdas[,3], x=c(.1, .2, .25, .5, .75, 1, 1.25, 1.5, 2, 2.5), type='l', lwd=2, xaxt='n', lty=3, ylim=c(1,2.5))
segments(x0=.1,x1=2.5, y0=1,y1=1, col=rgb(220/256,220/256,220/256,.5), lwd=2)



###################

par(las=1)
itt_difflambdas = res_summary[paste0('X.',seq(9, 36, by=3)),]

itt_difflambdas = itt_difflambdas[c(2:6,1,7:10),]

base = itt_difflambdas[6,]
itt_difflambdas = t(apply(itt_difflambdas, 1, function(x) x/base))

plot(itt_difflambdas[,1], x=c(.1, .2, .25, .5, .75, 1, 1.25, 1.5, 2, 2.5), type='l', lty=2, lwd=2, xaxt='n', ylim=c(1,2.5))
axis(side=1,at=c(.1, .5, .75, 1, 1.25, 1.5, 2, 2.5),labels=c(.1, .5, .75, 1, 1.25, 1.5, 2, 2.5))
mtext(expression(lambda/lambda[cv]),line=3,side=1,cex=1.5)
par(las=0)
mtext(expression(paste("Ratio of RMSEs for ATT for ", lambda," over ",lambda[cv])),
			line=2.5,side=2,cex=1.25)

#itt_difflambdas = res_summary[paste0('X.',seq(8, 36, by=3)),]
points(itt_difflambdas[,2], x=c(.1, .2, .25, .5, .75, 1, 1.25, 1.5, 2, 2.5), type='l', lwd=2, xaxt='n', lty=1, ylim=c(1,2.5))
points(itt_difflambdas[,3], x=c(.1, .2, .25, .5, .75, 1, 1.25, 1.5, 2, 2.5), type='l', lwd=2, xaxt='n', lty=3, ylim=c(1,2.5))
segments(x0=.1,x1=2.5, y0=1,y1=1, col=rgb(220/256,220/256,220/256,.5), lwd=2)

