
library(Synth)
library(augsynth)
library(osqp)
library(Matrix)


########################################
## simulation settings
#	similarclusseq for three clustering models for damping
#		= 2 for deterministically 
#		= 1 for probabilistially similar clusters
#		= 0 for probabilistially dissimilar clustering
## seedseq is used to track parallel run of the code

similarclusseq = c(0, 1, 2)
heteroseq = c(TRUE, FALSE)
seedseq = c(1234, 8041, 1546, 1362, 2315, 2141, 5362, 6163, 2326, 6126)
seedseq = c(seedseq, c(123, 804, 154, 162, 231, 241, 536, 163, 326, 126))
seedseq = c(seedseq, c(13, 84, 54, 62, 31, 41, 53, 63, 36, 26))
#seedseq = c(seedseq, c(3213, 584, 1254, 5363, 6431, 7841, 5573, 2363, 6436, 2726))
#seedseq = c(seedseq, c(8913, 9884, 8954, 9862, 8931, 9841, 8953, 9863, 8936, 9826))


foo <- function(x, y) c(x, y)
setup = outer(similarclusseq, heteroseq, FUN = Vectorize(foo, SIMPLIFY = FALSE))
setup = outer(setup, seedseq, FUN = Vectorize(foo, SIMPLIFY = FALSE))
setup = sapply(setup, cbind)

#####################################
## Estimation with L1 penalty 

for(id in 1:180){
	similarclus = setup[1,id]
	hetero = setup[2,id]==1
	thisseed = setup[3,id]
	set.seed(thisseed)

	## dismiss unnecessary printing
	deprintize<-function(f){
		 return(function(...) {capture.output(w<-f(...));return(w);});
	}

	options(width = 146)

	res_all = NULL
	for(itr in 1:15){
		#source('compare_methods_l1penalty_largesd.R')
		source('compare_methods_l1penalty.R')
		print(c(similarclus, hetero, res_sc, res_pooledsc, res_propscl1))
		#for(i in 1:length(lambdaseq))
		#	print(round(res_propsclambda[(i-1)*3+1:3]/res_propsc,2))
		res_all = rbind(res_all, c(res_sc, res_pooledsc, res_propscl1))
	}

	#write.csv(cbind(similarclus, hetero, thisseed, res_all), file = paste0('sc_sim_comparison_id_',id,'_null_revisionl1penalty_largesd.csv'))
	write.csv(cbind(similarclus, hetero, thisseed, res_all), file = paste0('sc_sim_comparison_id_',id,'_null_revisionl1penalty.csv'))

}


####################################
## Read results from file and summarizing 
#	the results
res = list()
for(set in 1:6)
	res[[set]] = matrix(0,ncol=13,nrow=0)

for(id in 1:180){
	
	similarclus = setup[1,id] + 1 
	hetero = 1+(setup[2,id]==1)
	set 	= 2*(similarclus-1)+ hetero
	print(set)
	f = 	paste0('sc_sim_comparison_id_',id,'_null_revisionl1penalty.csv')
	if(file.exists(f)){
		print('yes')
		temp = as.matrix(read.csv(f, header=TRUE))
		names(temp) = NULL
		res[[set]] = rbind(res[[set]], temp)
	}

}


res_summary = t( rbind(round(t(sapply(res, colMeans, na.rm=TRUE))[c(1,3,5),],2), 
round(t(sapply(res, function(x) apply(x, 2, function(y) sd(y, na.rm=TRUE)/sqrt(length(y)))))[c(1,3,5),],2)))#[1:40,]

res_summary = t( rbind(round(t(sapply(res, colMeans))[c(2,4,6),],2), 
round(t(sapply(res, function(x) apply(x, 2, function(y) sd(y)/sqrt(length(y)))))[c(2,4,6),],2)))


